#!/usr/bin/env python

# Generate gnarly test inputs for MeeshQuest Part 1.
# JK, 2007 Mar 29

import sys, random, math

# Goes out first: %u spatialWidth, %u spatialHeight
txt_header='<commands xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="part1in.xsd" spatialWidth="%u" spatialHeight="%u">'
# Goes out last
txt_footer='</commands>'
# createCity: %s name, %u x, %u y, %u radius, %s color
txt_createCity='<createCity name="%s" x="%u" y="%u" radius="%u" color="%s"/>'
# listCities sorted by name
txt_listCitiesByName='<listCities sortBy="name"/>'
# listCities sorted by coordinates
txt_listCitiesByCoords='<listCities sortBy="coordinate"/>'
# mapCity: %s name
txt_mapCity='<mapCity name="%s"/>'
# unmapCity: %s name
txt_unmapCity='<unmapCity name="%s"/>'
# deleteCity: %s name
txt_deleteCity='<deleteCity name="%s"/>'
# printPRQuadtree
txt_printPRQuadtree='<printPRQuadtree />'
# clearAll
txt_clearAll='<clearAll />'
#rangeCities: %u x, %u y, %u radius
txt_rangeCities='<rangeCities x="%u" y="%u" radius="%u"/>'
#nearestCity: %u x, %u y
txt_nearestCity='<nearestCity x="%u" y="%u"/>'


if len(sys.argv) != 2:
	sys.stderr.write("Generate gnarly testcases for MeeshQuest Part 1\n")
	sys.stderr.write("\tUsage:\tgnarly.py numcities\n")
	sys.exit(1)

# Number of cities to create
n = int(sys.argv[1])
# Spatial map width and height
width = 8192
height = 8192
# Fraction of created cities to immediately delete
delfrac = 0.1
# Number of rangeCities commands to give
num_rangeCities = 1000
# Number of nearestCity commands to give
num_nearestCity = 1000


# Header
print txt_header % (width, height)

# Create and map n cities, unmap and delete delfrac*n of them
for i in xrange(0,n):
	name = ''.join(random.sample("abcdefgh",6))
	x = random.randint(0,width-1)
	y = random.randint(0,height-1)
	radius = random.randint(0,100)
	color = random.choice(("red", "green", "blue", "yellow", "orange", "purple", "black"))
	print txt_createCity % (name, x, y, radius, color)
	print txt_mapCity % (name)
	
	if random.random() < delfrac:
		print txt_unmapCity % (name)
		print txt_deleteCity % (name)
		
# List cities by name, then coords, then dump quadtree
print txt_listCitiesByName
print txt_listCitiesByCoords
print txt_printPRQuadtree

# Do a bunch of rangeCities and nearestCity commands
for i in xrange(0,n):
	x = random.randint(0,width-1)
	y = random.randint(0,height-1)
	radius = random.randint(0,int(math.sqrt(width*height)))
	print txt_rangeCities % (x, y, radius)
	x = random.randint(0,width-1)
	y = random.randint(0,height-1)
	print txt_nearestCity % (x, y)
	
print txt_clearAll
print txt_footer

